@@ -1,16 +1,18 @@
 <?php
 // ** MySQL settings ** //
-define('DB_NAME', 'wordpress');    // The name of the database
-define('DB_USER', 'username');     // Your MySQL username
-define('DB_PASSWORD', 'password'); // ...and password
+define('DB_NAME', 'putyourdbnamehere');    // The name of the database
+define('DB_USER', 'usernamehere');     // Your MySQL username
+define('DB_PASSWORD', 'yourpasswordhere'); // ...and password
 define('DB_HOST', 'localhost');    // 99% chance you won't need to change this value
+define('DB_CHARSET', 'utf8');
+define('DB_COLLATE', '');
 
 // You can have multiple installations in one database if you give each a unique prefix
 $table_prefix  = 'wp_';   // Only numbers, letters, and underscores please!
 
 // Change this to localize WordPress.  A corresponding MO file for the
-// chosen language must be installed to wp-includes/languages.
-// For example, install de.mo to wp-includes/languages and set WPLANG to 'de'
+// chosen language must be installed to wp-content/languages.
+// For example, install de.mo to wp-content/languages and set WPLANG to 'de'
 // to enable German language support.
 define ('WPLANG', '');
 
@@ -18,4 +20,4 @@ define ('WPLANG', '');
 
 define('ABSPATH', dirname(__FILE__).'/');
 require_once(ABSPATH.'wp-settings.php');
-?>
\ No newline at end of file
+?>
