@@ -63,6 +63,7 @@ class WP_Object_Cache {
 	var $dirty_objects = array ();
 	var $non_existant_objects = array ();
 	var $global_groups = array ('users', 'userlogins', 'usermeta');
+	var $non_persistent_groups = array('comment');
 	var $blog_id;
 	var $cold_cache_hits = 0;
 	var $warm_cache_hits = 0;
@@ -186,16 +187,7 @@ class WP_Object_Cache {
 	}
 
 	function load_group_from_db($group) {
-		global $wpdb;
-
-		if ('category' == $group) {
-			$this->cache['category'] = array ();
-			if ($dogs = $wpdb->get_results("SELECT * FROM $wpdb->categories")) {
-				foreach ($dogs as $catt)
-					$this->cache['category'][$catt->cat_ID] = $catt;
-			}
-		}
-
+		return;
 	}
 
 	function make_group_dir($group, $perms) {
@@ -317,6 +309,9 @@ class WP_Object_Cache {
 		// Loop over dirty objects and save them.
 		$errors = 0;
 		foreach ($this->dirty_objects as $group => $ids) {
+			if ( in_array($group, $this->non_persistent_groups) )
+				continue;
+
 			$group_dir = $this->make_group_dir($group, $dir_perms);
 
 			$ids = array_unique($ids);
@@ -360,14 +355,14 @@ class WP_Object_Cache {
 
 	function stats() {
 		echo "<p>";
-		echo "<strong>Cold Cache Hits:</strong> {$this->cold_cache_hits}<br/>";
-		echo "<strong>Warm Cache Hits:</strong> {$this->warm_cache_hits}<br/>";
-		echo "<strong>Cache Misses:</strong> {$this->cache_misses}<br/>";
+		echo "<strong>Cold Cache Hits:</strong> {$this->cold_cache_hits}<br />";
+		echo "<strong>Warm Cache Hits:</strong> {$this->warm_cache_hits}<br />";
+		echo "<strong>Cache Misses:</strong> {$this->cache_misses}<br />";
 		echo "</p>";
 
 		foreach ($this->cache as $group => $cache) {
 			echo "<p>";
-			echo "<strong>Group:</strong> $group<br/>";
+			echo "<strong>Group:</strong> $group<br />";
 			echo "<strong>Cache:</strong>";
 			echo "<pre>";
 			print_r($cache);
