@@ -1,7 +1,7 @@
 <?php
 
 if (empty($wp)) {
-	require_once('wp-config.php');
+	require_once('./wp-config.php');
 	wp('tb=1');
 }
 
@@ -66,7 +66,7 @@ if (empty($title) && empty($tb_url) && empty($blog_name)) {
 	exit;
 }
 
-if ( !empty($tb_url) && !empty($title) && !empty($tb_url) ) {
+if ( !empty($tb_url) && !empty($title) ) {
 	header('Content-Type: text/xml; charset=' . get_option('blog_charset') );
 
 	$pingstatus = $wpdb->get_var("SELECT ping_status FROM $wpdb->posts WHERE ID = $tb_id");
